-- ===========================================================================
-- Calc is a calculator that can add, subtract, multiply, and divide.
-- Copyright (C) 2016-2016 Eric T. Hughes (a.k.a. "Eboneezur")
--
--  This program is free software: you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program.  If not, see <http://www.gnu.org/licenses/>.
-- ===========================================================================
-- ===========================================================================
--- Class TPower
--
-- This class does little.  Not sure why I thought I needed to simulate a 
-- power button for a calculator.
-- ===========================================================================
do
  local nIsOnce = 0
  local nIsOn   = 1

-- -------------------------------------------------------------------------
--- Method ON
--
-- This method turns on the calculator
-- -------------------------------------------------------------------------
  local function ON()
    CalcFrameText:SetText("Calc ON")
    nIsOn = 1
    DEFAULT_CHAT_FRAME:AddMessage("[Calc] says: Power is turned on.", 0.0, 0.85, 0.0)

    if (nil ~= Broker_ACalc.obj) then
      Broker_ACalc.obj.icon = "Interface\\Addons\\ACalc\\images\\bit_yes"
      Broker_ACalc.obj.label = "Calc"
      Broker_ACalc.obj.text = "ON"
    end
  end

-- -------------------------------------------------------------------------
--- Method OFF
--
-- This method turns off the calculator
-- -------------------------------------------------------------------------
  local function OFF()
    CalcFrameText:SetText("ACalc OFF")
    nIsOn = 0
    DEFAULT_CHAT_FRAME:AddMessage("[ACalc] says: Power is turned off.", 0.0, 0.85, 0.0)

    if (nil ~= Broker_ACalc.obj) then
      Broker_ACalc.obj.icon = "Interface\\Addons\\ACalc\\images\\bit_default"
      Broker_ACalc.obj.label = "ACalc"
      Broker_ACalc.obj.text = "OFF"
    end
  end
  
-- -------------------------------------------------------------------------
--- Method HIDE
--
-- This method hides the calculator
-- -------------------------------------------------------------------------
  local function HIDE()
    if (nil ~= Broker_ACalc.obj) then
      Broker_ACalc.obj.icon = "Interface\\Addons\\ACalc\\images\\bit_default"
      Broker_ACalc.obj.label = "ACalc"
      Broker_ACalc.obj.text = "Hide"
    end
  end

  
-- -------------------------------------------------------------------------
--- Method IsOn
--
-- This method returns 1 when the power is on or 0 when the power is off.
-- -------------------------------------------------------------------------
  local function IsOn()
    return nIsOn
  end

-- -------------------------------------------------------------------------
--- Method Toggle
--
-- This method determines whether to turn on or off the power.
-- @param self The given TPower object.
-- -------------------------------------------------------------------------
  local function Toggle(self)
    self.m_vecLocation.nIsInInstance, 
    self.m_vecLocation.strInstanceType = IsInInstance()

    if   (1      == self.m_vecLocation.nIsInInstance)   then
      if ("raid" == self.m_vecLocation.strInstanceType) then
        self.ON()
      else
        if (1 == nIsOn) then
          self.OFF()
        end
      end
    else
      if (1 == nIsOn) then
        CalcFrameText:SetText("ACalc OFF")

        if   (1 == nIsOnce) then
        else
          DEFAULT_CHAT_FRAME:AddMessage(string.format("[Calc] says: Type /acalc show, /acalc hide, or /acalc config."), 0.0, 0.85, 0.0)
          nIsOnce = 1
        end

        nIsOn = 0
        DEFAULT_CHAT_FRAME:AddMessage("[ACalc] says: The calculator is hidden.", 0.0, 0.85, 0.0)
      end
    end

  end

-- -------------------------------------------------------------------------
--- Method Update
--
-- This method Updates the zone.
-- @param self The given TPower object.
-- -------------------------------------------------------------------------
  local function Update(self)
    self.m_vecLocation.strZone    = GetZoneText()
    self.m_vecLocation.strSubZone = GetSubZoneText()
    SetMapToCurrentZone()
  end


-- -------------------------------------------------------------------------
--- Method GetType
--
-- This method returns the type of this class.
-- -------------------------------------------------------------------------
  local function GetType()
      return "TPower"
   end

-- -------------------------------------------------------------------------
-- Method new_TPower
--
-- This method creates an object of this class.
-- -------------------------------------------------------------------------
  function new_TPower()

      local obj =
      {
        GetType = GetType,
        IsOn    = IsOn,
        OFF     = OFF,
        ON      = ON,
		HIDE    = HIDE,
        Toggle  = Toggle,
        Update  = Update,

        m_vecLocation =
        {
          strInstanceType = "",
          strZone         = "",
          strSubZone      = "",
          nX              = 0.0,
          nY              = 0.0,
          nIsInInstance   = 1,
        },
      }

      return obj
  end 
end


-- ===========================================================================
--- Class TSemaphore
--
-- This class defines a counting semaphore.
-- ===========================================================================
do
-- -------------------------------------------------------------------------
--- Method IsLocked
--
-- This method returns true if this counting semaphore has a lock.
-- @param self The given TSemaphore object.
-- -------------------------------------------------------------------------
  local function IsLocked(self)
    if 1 > self.nLockCount then
      return false
    else
      return true
    end
  end

-- -------------------------------------------------------------------------
--- Method IsUnLocked
--
-- This method returns true if this counting semaphore does not have a lock.
-- @param self The given TSemaphore object.
-- -------------------------------------------------------------------------
  local function IsUnLocked(self)
    if 0 < self.nLockCount then
      return false
    else
      return true
    end
  end

-- -------------------------------------------------------------------------
--- Method Lock
--
-- This method locks this counting semaphore and increases the lock count.
-- @param self The given TSemaphore object.
-- -------------------------------------------------------------------------
  local function Lock(self)
    self.nLockCount = self.nLockCount + 1
  end

-- -------------------------------------------------------------------------
--- Method UnLock
--
-- This method unlocks this counting semaphore and decreases the lock count.
-- @param self The given TSemaphore object.
-- -------------------------------------------------------------------------
  local function UnLock(self)
    if (0 < self.nLockCount) then
      self.nLockCount = self.nLockCount - 1
    end
  end

-- -------------------------------------------------------------------------
--- Method GetType
--
-- This method returns the type of this class.
-- -------------------------------------------------------------------------
  local function GetType()
    return "TSemaphore"
  end

-- -------------------------------------------------------------------------
--- Method new_TSemaphore
--
-- This method creates an object of this class.
-- -------------------------------------------------------------------------
  function new_TSemaphore()
    local obj =
    {
      GetType    = GetType,
      IsLocked   = IsLocked,
      IsUnLocked = IsUnLocked,
      Lock       = Lock,
      UnLock     = UnLock,
      nLockCount = 0,
    }

    return obj
  end 
end



-- =========================================================================
-- Class TDialogCalculator
--
-- This class defines the calculator's user interface.
-- =========================================================================
do -- Class TDialogCalculator
  -- -------------------------------------------------------------------------
  --- Method SetDecimalPoint
  --
  -- This method appends a decimal point in the text box.
  -- -------------------------------------------------------------------------
  local function SetDecimalPoint()
    CalculatorDialog_TextBox:SetText(string.format("%s.", CalculatorDialog_TextBox:GetText()))
  end
  
  -- -------------------------------------------------------------------------
  --- Method SetText
  --
  -- This method adds the given text to the text box.
  -- -------------------------------------------------------------------------
  local function SetText(a_strText)
    CalculatorDialog_TextBox:SetText(a_strText)
  end
  
  -- -------------------------------------------------------------------------
  --- Method Reset
  --
  -- This method clears the text box and resets the calculator.
  -- -------------------------------------------------------------------------
  local function Reset(self)
    CalculatorDialog_TextBox:SetText("")
	self.tblCalculator:Reset()
  end
  
  -- -------------------------------------------------------------------------
  --- Method SetOperand
  --
  -- This method appends the given operand to the expression in the text box.
  -- -------------------------------------------------------------------------
  local function SetOperand(a_numOperand)
    CalculatorDialog_TextBox:SetText(string.format("%s%s", CalculatorDialog_TextBox:GetText(), a_numOperand))
  end
  
  -- -------------------------------------------------------------------------
  --- Method SetOperator
  --
  -- This method evaluates the given operator and updates the text box.
  -- -------------------------------------------------------------------------
  local function SetOperator(self, a_strOperator)
    local numResult = nil
	local strPlusSign
	local strMinusSign
	local strMultiplySign
	local strDivideSign
	local strPowerSign
	local strPlusMinus
	local strTextForAbsoluteValue
	local strTextForArcCosine
	local strTextForArcSine
	local strTextForArcTangent
	local strTextForArcTangent2
	local strTextForCeiling
	local strTextForHyperbolicCosine
	local strTextForCosine
	local strTextForDegrees
	local strTextForExp
	local strTextForFloor
	local strTextForFMod
	local strTextForBase10Logarithm
	local strTextForNaturalLogarithm
	local strTextForMaximum
	local strTextForMinimum
	local strTextForPi
	local strTextForPower
	local strTextForRadians
	local strTextForHyperbolicSine
	local strTextForSine
	local strTextForSquareRoot
	local strTextForHyperbolicTangent
	local strTextForTangent
	
	strPlusSign                 = self.tblCalculator.GetPlusSign()
	strMinusSign                = self.tblCalculator.GetMinusSign()
	strMultiplySign             = self.tblCalculator.GetMultiplySign()
	strDivideSign               = self.tblCalculator.GetDivideSign()
	strPowerSign                = self.tblCalculator.GetPowerSign()
	strTextForAbsoluteValue     = self.tblCalculator.tblGetTexts.GetTextForAbsoluteValue()
	strTextForArcCosine         = self.tblCalculator.tblGetTexts.GetTextForArcCosine()
	strTextForArcSine           = self.tblCalculator.tblGetTexts.GetTextForArcSine()
	strTextForArcTangent        = self.tblCalculator.tblGetTexts.GetTextForArcTangent()
	strTextForArcTangent2       = self.tblCalculator.tblGetTexts.GetTextForArcTangent2()
	strTextForCeiling           = self.tblCalculator.tblGetTexts.GetTextForCeiling()
	strTextForHyperbolicCosine  = self.tblCalculator.tblGetTexts.GetTextForHyperbolicCosine()
	strTextForCosine            = self.tblCalculator.tblGetTexts.GetTextForCosine()
	strTextForDegrees           = self.tblCalculator.tblGetTexts.GetTextForDegrees()	
	strTextForExp               = self.tblCalculator.tblGetTexts.GetTextForExp()
	strTextForFloor             = self.tblCalculator.tblGetTexts.GetTextForFloor()
	strTextForFMod              = self.tblCalculator.tblGetTexts.GetTextForFMod()
	strTextForBase10Logarithm   = self.tblCalculator.tblGetTexts.GetTextForBase10Logarithm()
	strTextForNaturalLogarithm  = self.tblCalculator.tblGetTexts.GetTextForNaturalLogarithm()
	strTextForMaximum           = self.tblCalculator.tblGetTexts.GetTextForMaximum()
	strTextForMinimum           = self.tblCalculator.tblGetTexts.GetTextForMinimum()
	strTextForPi                = self.tblCalculator.tblGetTexts.GetTextForPi()
	strTextForPower             = self.tblCalculator.tblGetTexts.GetTextForPower()
	strTextForRadians           = self.tblCalculator.tblGetTexts.GetTextForRadians()
	strTextForHyperbolicSine    = self.tblCalculator.tblGetTexts.GetTextForHyperbolicSine()
	strTextForSine              = self.tblCalculator.tblGetTexts.GetTextForSine()
	strTextForSquareRoot        = self.tblCalculator.tblGetTexts.GetTextForSquareRoot()
	strTextForHyperbolicTangent = self.tblCalculator.tblGetTexts.GetTextForHyperbolicTangent()
	strTextForTangent           = self.tblCalculator.tblGetTexts.GetTextForTangent()
	
	
    if (strPlusSign == a_strOperator) then 
	  numResult = self.tblCalculator:Add()
	end
	
	if (strMinusSign == a_strOperator) then 
	  numResult = self.tblCalculator:Subtract() 
	end
	
	if (strMultiplySign == a_strOperator) then 
	  numResult = self.tblCalculator:Multiply() 
	end
	
	if (strDivideSign == a_strOperator) then 
	  numResult = self.tblCalculator:Divide()
	end
	
	if (strPowerSign == a_strOperator) then
	  numResult = self.tblCalculator:Power()
	end
	
	if (strTextForAbsoluteValue == a_strOperator) then
	  numResult = self.tblCalculator:AbsoluteValue()
	end

	if (strTextForArcCosine == a_strOperator) then
	  numResult = self.tblCalculator:ArcCosine()
	end

	if (strTextForArcSine == a_strOperator) then
	  numResult = self.tblCalculator:ArcSine()
	end
	
	if (strTextForArcTangent == a_strOperator) then
	  numResult = self.tblCalculator:ArcTangent()
	end
	
	if (strTextForArcTangent2 == a_strOperator) then
	  numResult = self.tblCalculator:ArcTangent2()
	end

	if (strTextForCeiling == a_strOperator) then
	  numResult = self.tblCalculator:Ceiling()
	end

	if (strTextForHyperbolicCosine == a_strOperator) then
	  numResult = self.tblCalculator:HyperbolicCosine()
	end

	if (strTextForCosine == a_strOperator) then
	  numResult = self.tblCalculator:Cosine()
	end

	if (strTextForDegrees == a_strOperator) then
	  numResult = self.tblCalculator:Degrees()
	end

	if (strTextForExp == a_strOperator) then
	  numResult = self.tblCalculator:Exp()
	end

	if (strTextForFloor == a_strOperator) then
	  numResult = self.tblCalculator:Floor()
	end

	if (strTextForFMod == a_strOperator) then
	  numResult = self.tblCalculator:FMod()
	end

	if (strTextForBase10Logarithm == a_strOperator) then
	  numResult = self.tblCalculator:Base10Logarithm()
	end

	if (strTextForNaturalLogarithm == a_strOperator) then
	  numResult = self.tblCalculator:NaturalLogarithm()
	end

	if (strTextForMaximum == a_strOperator) then
	  numResult = self.tblCalculator:Maximum()
	end

	if (strTextForMinimum == a_strOperator) then
	  numResult = self.tblCalculator:Minimum()
	end

	if (strTextForPi == a_strOperator) then
	  numResult = self.tblCalculator:Pi()
	end

	if (strTextForPower == a_strOperator) then
	  numResult = self.tblCalculator:Power()
	end

	if (strTextForRadians == a_strOperator) then
	  numResult = self.tblCalculator:Radians()
	end

	if (strTextForHyperbolicSine == a_strOperator) then
	  numResult = self.tblCalculator:HyperbolicSine()
	end

	if (strTextForSine == a_strOperator) then
	  numResult = self.tblCalculator:Sine()
	end

	if (strTextForSquareRoot == a_strOperator) then
	  numResult = self.tblCalculator:SquareRoot()
	end

	if (strTextForHyperbolicTangent == a_strOperator) then
	  numResult = self.tblCalculator:HyperbolicTangent()
	end

	if (strTextForTangent == a_strOperator) then
	  numResult = self.tblCalculator:Tangent()
	end
	
	
	if (nil~=numResult) then
      CalculatorDialog_TextBox:SetText(string.format("%s", numResult))
	end
  end

  do -- Class TDialogCalculator::TButtonZero
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent.SetText(self.tblParent.tblCalculator:AppendDigit0())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonZero"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonZero
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonZero()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do  -- Class TDialogCalculator::TButtonDecimalPoint
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent.SetText(self.tblParent.tblCalculator:AppendDecimalPoint())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonDecimalPoint"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonDecimalPoint
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonDecimalPoint()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do  -- Class TDialogCalculator::TButtonPlusMinus
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent.SetText(self.tblParent.tblCalculator:ToggleSign())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonPlusMinus"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonPlusMinus
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonPlusMinus()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end
  
  do -- Class TDialogCalculator::TButtonEqualSign
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
     self.tblParent.tblCalculator:SetValue()	
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonEqualSign"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonEqualSign
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonEqualSign()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonPlusSign
  -- -----------------------------------numValue1-----------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator:GetPlusSign())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonPlusSign"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonPlusSign
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonPlusSign()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonOne
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent.SetText(self.tblParent.tblCalculator:AppendDigit1())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonOne"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonOne
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonOne()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonTwo
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent.SetText(self.tblParent.tblCalculator:AppendDigit2())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonTwo"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonTwo
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonTwo()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonThree
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent.SetText(self.tblParent.tblCalculator:AppendDigit3())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonThree"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonThree
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonThree()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonMinusSign
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator:GetMinusSign())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonMinusSign"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonMinusSign
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonMinusSign()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonFour
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent.SetText(self.tblParent.tblCalculator:AppendDigit4())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonFour"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonFour
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonFour()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonFive
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent.SetText(self.tblParent.tblCalculator:AppendDigit5())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonFive"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonFive
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonFive()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonSix
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent.SetText(self.tblParent.tblCalculator:AppendDigit6())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonSix"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonSix
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonSix()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonMultiplySign
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator:GetMultiplySign())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonMultiplySign"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonMultiplySign
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonMultiplySign()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonSeven
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent.SetText(self.tblParent.tblCalculator:AppendDigit7())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonSeven"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonSeven
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonSeven()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonEight
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent.SetText(self.tblParent.tblCalculator:AppendDigit8())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonEight"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonEight
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonEight()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- TDialogCalculator::TButtonNine
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent.SetText(self.tblParent.tblCalculator:AppendDigit9())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonNine"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonNine
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonNine()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonDivideSign
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator.GetDivideSign())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonDivideSign"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonDivideSign
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonDivideSign()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonPowerSign
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator.GetPowerSign())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonPowerSign"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonPowerSign
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonPowerSign()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end
  
  do -- Class TDialogCalculator::TButtonMana
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  local numMana
	  local strMana
	
	  numMana = UnitPowerMax('player')
	  strMana = string.format("%s", numMana)
	  self.tblParent.tblCalculator.strValue = strMana
	  self.tblParent.tblCalculator:SetValue()
	  self.tblParent.SetText(strMana)
	  
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonMana"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonMana
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonMana()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonGold
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  local numGold
	  local strGold
	
	  numGold = GetMoney() / 10000
	  strGold = string.format("%s", numGold)
	  self.tblParent.tblCalculator.strValue = strGold
	  self.tblParent.tblCalculator:SetValue()
	  self.tblParent.SetText(strGold)
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonGold"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonGold
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonGold()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonSilver
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  local numSilver
	  local strSilver
	
	  numSilver = GetMoney() / 100
	  strSilver = string.format("%s", numSilver)
	  self.tblParent.tblCalculator.strValue = strSilver
	  self.tblParent.tblCalculator:SetValue()
	  self.tblParent.SetText(strSilver)
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonSilver"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonSilver
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonSilver()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonCopper
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  local numCopper
	  local strCopper
	
	  numCopper = GetMoney()
	  strCopper = string.format("%s", numCopper)
	  self.tblParent.tblCalculator.strValue = strCopper
	  self.tblParent.tblCalculator:SetValue()
	  self.tblParent.SetText(strCopper)
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonCopper"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonCopper
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonCopper()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonHealth
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  local numHealth
	  local strHealth
	
	  numHealth = UnitHealthMax('player')
	  strHealth = string.format("%s", numHealth)
	  self.tblParent.tblCalculator.strValue = strHealth
	  self.tblParent.tblCalculator:SetValue()
	  self.tblParent.SetText(strHealth)
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonHealth"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonHealth
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonHealth()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonAchieves
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  local numAchievementPoints
	  local strAchievementPoints
	
	  numAchievementPoints = GetTotalAchievementPoints()
	  strAchievementPoints = string.format("%s", numAchievementPoints)
	  self.tblParent.tblCalculator.strValue = strAchievementPoints
	  self.tblParent.tblCalculator:SetValue()
	  self.tblParent.SetText(strAchievementPoints)
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonAchieves"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonAchieves
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonAchieves()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonConquest
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
      RequestPVPRewards()
    end
  
    local function OnClick(self)
	  local numConquest
	  local strConquest
	
	  numConquest = select(2,GetCurrencyInfo(390)) or 0	
	  strConquest = string.format("%s", numConquest)
	  self.tblParent.tblCalculator.strValue = strConquest
	  self.tblParent.tblCalculator:SetValue()
	  self.tblParent.SetText(strConquest)
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonConquest"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonConquest
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonConquest()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonHonor
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  local numHonor
	  local strHonor
	
	  numHonor = select(2,GetCurrencyInfo(392)) or 0
	  strHonor = string.format("%s", numHonor)
	  self.tblParent.tblCalculator.strValue = strHonor
	  self.tblParent.tblCalculator:SetValue()
	  self.tblParent.SetText(strHonor)
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonHonor"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonHonor
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonHonor()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonJustice
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  local numJustice
	  local strJustice
	
	  numJustice = select(2,GetCurrencyInfo(395)) or 0
	  strJustice = string.format("%s", numJustice)
	  self.tblParent.tblCalculator.strValue = strJustice
	  self.tblParent.tblCalculator:SetValue()
	  self.tblParent.SetText(strJustice)
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonJustice"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonJustice
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonJustice()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonValor
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  local numValor
	  local strValor
	
	  numValor = select(2,GetCurrencyInfo(396)) or 0
	  strValor = string.format("%s", numValor)
	  self.tblParent.tblCalculator.strValue = strValor
	  self.tblParent.tblCalculator:SetValue()
	  self.tblParent.SetText(strValor)
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonValor"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonValor
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonValor()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonVPCap
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  local numValorPointWeeklyCap
	  local strValorPointWeeklyCap
	
	  numValorPointWeeklyCap = select(5,GetCurrencyInfo(396))/100 - select(4,GetCurrencyInfo(396)) or 0
	  strValorPointWeeklyCap = string.format("%s", numValorPointWeeklyCap)
	  self.tblParent.tblCalculator.strValue = strValorPointWeeklyCap
	  self.tblParent.tblCalculator:SetValue()
	  self.tblParent.SetText(strValorPointWeeklyCap)
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonVPCap"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonVPCap
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonVPCap()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

    return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonAbsoluteValue
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator.tblGetTexts.GetTextForAbsoluteValue())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonAbsoluteValue"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonAbsoluteValue
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonAbsoluteValue()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonArcCosine
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator.tblGetTexts.GetTextForArcCosine())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonArcCosine"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonArcCosine
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonArcCosine()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonArcSine
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator.tblGetTexts.GetTextForArcSine())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonArcSine"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonArcSine
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonArcSine()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonArcTangent
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator.tblGetTexts.GetTextForArcTangent())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonArcTangent"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonArcTangent
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonArcTangent()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonArcTangent2
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator.tblGetTexts.GetTextForArcTangent2())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonArcTangent2"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonArcTangent2
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonArcTangent2()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonCeiling
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator.tblGetTexts.GetTextForCeiling())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonCeiling"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonCeiling
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonCeiling()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonCosine
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator.tblGetTexts.GetTextForCosine())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonCosine"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonCosine
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonCosine()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonHyperbolicCosine
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator.tblGetTexts.GetTextForHyperbolicCosine())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonHyperbolicCosine"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonHyperbolicCosine
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonHyperbolicCosine()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end
  
  do -- Class TDialogCalculator::TButtonDegrees
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator.tblGetTexts.GetTextForDegrees())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonDegrees"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonDegrees
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonDegrees()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonExp
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator.tblGetTexts.GetTextForExp())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonExp"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonExp
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonExp()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonFloor
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator.tblGetTexts.GetTextForFloor())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonFloor"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonFloor
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonFloor()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonFMod
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator.tblGetTexts.GetTextForFMod())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonFMod"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonFMod
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonFMod()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonNaturalLogarithm
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator.tblGetTexts.GetTextForNaturalLogarithm())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonNaturalLogarithm"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonNaturalLogarithm
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonNaturalLogarithm()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonBase10Logarithm
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator.tblGetTexts.GetTextForBase10Logarithm())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonBase10Logarithm"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonBase10Logarithm
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonBase10Logarithm()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonMaximum
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator.tblGetTexts.GetTextForMaximum())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonMaximum"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonMaximum
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonMaximum()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonMinimum
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator.tblGetTexts.GetTextForMinimum())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonMinimum"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonMinimum
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonMinimum()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonPi
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator.tblGetTexts.GetTextForPi())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonPi"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonPi
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonPi()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonPower
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator.tblGetTexts.GetTextForPower())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonPower"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonPower
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonPower()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonRadians
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator.tblGetTexts.GetTextForRadians())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonRadians"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonRadians
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonRadians()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonSine
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator.tblGetTexts.GetTextForSine())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonSine"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonSine
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonSine()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonHyperbolicSine
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator.tblGetTexts.GetTextForHyperbolicSine())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonHyperbolicSine"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonHyperbolicSine
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonHyperbolicSine()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonSquareRoot
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator.tblGetTexts.GetTextForSquareRoot())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonSquareRoot"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonSquareRoot
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonSquareRoot()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonTangent
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator.tblGetTexts.GetTextForTangent())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonTangent"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonTangent
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonTangent()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonHyperbolicTangent
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator.tblGetTexts.GetTextForHyperbolicTangent())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonHyperbolicTangent"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonHyperboicTangent
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonHyperbolicTangent()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end
  
  
  do -- Class TDialogCalculator::TButtonReset
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:Reset()
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonReset"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonReset
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonReset()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

    return obj
    end 
  end
  
-- -------------------------------------------------------------------------
--- Method GetType
--
-- This method returns the type of this class.
-- -------------------------------------------------------------------------
  local function GetType()
    return "TDialogCalculator"
  end
-- -------------------------------------------------------------------------
--- Method new_TDialogCalculator
--
-- This method creates an object of this class.
-- -------------------------------------------------------------------------
  function new_TDialogCalculator()
    local objDialogCalculator =
    {
      GetType                    = GetType,
	  Reset                      = Reset,
	  SetDecimalPoint            = SetDecimalPoint,
	  SetText                    = SetText,
	  SetOperand                 = SetOperand,
	  SetOperator                = SetOperator,
      tblButtonZero              = new_TButtonZero(),
      tblButtonDecimalPoint      = new_TButtonDecimalPoint(),
	  tblButtonPlusMinus         = new_TButtonPlusMinus(),
      tblButtonEqualSign         = new_TButtonEqualSign(),
      tblButtonPlusSign          = new_TButtonPlusSign(),
      tblButtonOne               = new_TButtonOne(),
      tblButtonTwo               = new_TButtonTwo(),
      tblButtonThree             = new_TButtonThree(),
      tblButtonMinusSign         = new_TButtonMinusSign(),
      tblButtonFour              = new_TButtonFour(),
      tblButtonFive              = new_TButtonFive(),
      tblButtonSix               = new_TButtonSix(),
      tblButtonMultiplySign      = new_TButtonMultiplySign(),
      tblButtonSeven             = new_TButtonSeven(),
      tblButtonEight             = new_TButtonEight(),
      tblButtonNine              = new_TButtonNine(),
      tblButtonDivideSign        = new_TButtonDivideSign(),
	  tblButtonPowerSign         = new_TButtonPowerSign(),
      tblButtonMana              = new_TButtonMana(),
      tblButtonGold              = new_TButtonGold(),
      tblButtonSilver            = new_TButtonSilver(),
      tblButtonCopper            = new_TButtonCopper(),
      tblButtonHealth            = new_TButtonHealth(),
      tblButtonAchieves          = new_TButtonAchieves(),
      tblButtonConquest          = new_TButtonConquest(),
      tblButtonHonor             = new_TButtonHonor(),
      tblButtonJustice           = new_TButtonJustice(),
      tblButtonValor             = new_TButtonValor(),
      tblButtonVPCap             = new_TButtonVPCap(),
	  tblButtonReset             = new_TButtonReset(),
	  tblButtonAbsoluteValue     = new_TButtonAbsoluteValue(),
	  tblButtonArcCosine         = new_TButtonArcCosine(),
	  tblButtonArcSine           = new_TButtonArcSine(),
	  tblButtonArcTangent        = new_TButtonArcTangent(),
	  tblButtonArcTangent2       = new_TButtonArcTangent2(),
	  tblButtonCeiling           = new_TButtonCeiling(),
	  tblButtonCosine            = new_TButtonCosine(),
	  tblButtonHyperbolicCosine  = new_TButtonHyperbolicCosine(),
	  tblButtonDegrees           = new_TButtonDegrees(),
	  tblButtonExp               = new_TButtonExp(),
	  tblButtonFloor             = new_TButtonFloor(),
	  tblButtonFMod              = new_TButtonFMod(),
	  tblButtonNaturalLogarithm  = new_TButtonNaturalLogarithm(),
	  tblButtonBase10Logarithm   = new_TButtonBase10Logarithm(),
	  tblButtonMaximum           = new_TButtonMaximum(),
	  tblButtonMinimum           = new_TButtonMinimum(),
	  tblButtonPi                = new_TButtonPi(),
	  tblButtonPower             = new_TButtonPower(),
	  tblButtonRadians           = new_TButtonRadians(),
	  tblButtonSine              = new_TButtonSine(),
	  tblButtonHyperbolicSine    = new_TButtonHyperbolicSine(),
	  tblButtonSquareRoot        = new_TButtonSquareRoot(),
	  tblButtonTangent           = new_TButtonTangent(),
	  tblButtonHyperbolicTangent = new_TButtonHyperbolicTangent(),
	  tblCalculator              = new_TModelCalculator(),
    }

    objDialogCalculator.tblButtonZero.tblParent              = objDialogCalculator		
	objDialogCalculator.tblButtonDecimalPoint.tblParent      = objDialogCalculator
    objDialogCalculator.tblButtonPlusMinus.tblParent         = objDialogCalculator	
    objDialogCalculator.tblButtonEqualSign.tblParent         = objDialogCalculator		
    objDialogCalculator.tblButtonPlusSign.tblParent          = objDialogCalculator	
    objDialogCalculator.tblButtonOne.tblParent               = objDialogCalculator	
    objDialogCalculator.tblButtonTwo.tblParent               = objDialogCalculator		
    objDialogCalculator.tblButtonThree.tblParent             = objDialogCalculator		
    objDialogCalculator.tblButtonMinusSign.tblParent         = objDialogCalculator	
    objDialogCalculator.tblButtonFour.tblParent              = objDialogCalculator		
    objDialogCalculator.tblButtonFive.tblParent              = objDialogCalculator	
    objDialogCalculator.tblButtonSix.tblParent               = objDialogCalculator		
    objDialogCalculator.tblButtonMultiplySign.tblParent      = objDialogCalculator	
    objDialogCalculator.tblButtonSeven.tblParent             = objDialogCalculator	
    objDialogCalculator.tblButtonEight.tblParent             = objDialogCalculator	
    objDialogCalculator.tblButtonNine.tblParent              = objDialogCalculator		
    objDialogCalculator.tblButtonDivideSign.tblParent        = objDialogCalculator	
	objDialogCalculator.tblButtonPowerSign.tblParent         = objDialogCalculator
    objDialogCalculator.tblButtonMana.tblParent              = objDialogCalculator	
    objDialogCalculator.tblButtonGold.tblParent              = objDialogCalculator		
    objDialogCalculator.tblButtonSilver.tblParent            = objDialogCalculator		
    objDialogCalculator.tblButtonCopper.tblParent            = objDialogCalculator		
    objDialogCalculator.tblButtonHealth.tblParent            = objDialogCalculator	
	objDialogCalculator.tblButtonAchieves.tblParent          = objDialogCalculator	
    objDialogCalculator.tblButtonConquest.tblParent          = objDialogCalculator		
	objDialogCalculator.tblButtonHonor.tblParent             = objDialogCalculator	
	objDialogCalculator.tblButtonJustice.tblParent           = objDialogCalculator	
	objDialogCalculator.tblButtonValor.tblParent             = objDialogCalculator	
	objDialogCalculator.tblButtonVPCap.tblParent             = objDialogCalculator
	objDialogCalculator.tblButtonAbsoluteValue.tblParent     = objDialogCalculator
	objDialogCalculator.tblButtonArcCosine.tblParent         = objDialogCalculator
	objDialogCalculator.tblButtonArcSine.tblParent           = objDialogCalculator
	objDialogCalculator.tblButtonArcTangent.tblParent        = objDialogCalculator
	objDialogCalculator.tblButtonArcTangent2.tblParent       = objDialogCalculator
	objDialogCalculator.tblButtonCeiling.tblParent           = objDialogCalculator
	objDialogCalculator.tblButtonCosine.tblParent            = objDialogCalculator
	objDialogCalculator.tblButtonHyperbolicCosine.tblParent  = objDialogCalculator
	objDialogCalculator.tblButtonDegrees.tblParent           = objDialogCalculator
	objDialogCalculator.tblButtonExp.tblParent               = objDialogCalculator
	objDialogCalculator.tblButtonFloor.tblParent             = objDialogCalculator
	objDialogCalculator.tblButtonFMod.tblParent              = objDialogCalculator
	objDialogCalculator.tblButtonNaturalLogarithm.tblParent  = objDialogCalculator
	objDialogCalculator.tblButtonBase10Logarithm.tblParent   = objDialogCalculator
	objDialogCalculator.tblButtonMaximum.tblParent           = objDialogCalculator
	objDialogCalculator.tblButtonMinimum.tblParent           = objDialogCalculator
	objDialogCalculator.tblButtonPi.tblParent                = objDialogCalculator
	objDialogCalculator.tblButtonPower.tblParent             = objDialogCalculator
	objDialogCalculator.tblButtonRadians.tblParent           = objDialogCalculator
	objDialogCalculator.tblButtonSine.tblParent              = objDialogCalculator
	objDialogCalculator.tblButtonHyperbolicSine.tblParent    = objDialogCalculator
	objDialogCalculator.tblButtonSquareRoot.tblParent        = objDialogCalculator
	objDialogCalculator.tblButtonTangent.tblParent           = objDialogCalculator
	objDialogCalculator.tblButtonHyperbolicTangent.tblParent = objDialogCalculator
	objDialogCalculator.tblButtonReset.tblParent             = objDialogCalculator		

    return objDialogCalculator
  end 
end

tblDialogCalculator = new_TDialogCalculator()


-- ===========================================================================
--- Class TCalc
--
-- This class encapsulates the WoW API.
-- ===========================================================================
do
  -- Protect event handlers against WoW patches and programmer error.

  local semEventPLAYER_ENTERING_WORLD = new_TSemaphore()
  local nIsDisplayed = 0  -- Default to hidden

-- -------------------------------------------------------------------------
--- Method Hide
--
-- This method hides the calculator.
-- @param self The given TCalc object.
-- -------------------------------------------------------------------------
   local function Hide(self)
      CalcFrame:Hide()
	  CalculatorDialog:Hide()
      nIsDisplayed = 0
   end

-- -------------------------------------------------------------------------
--- Method IsDisplayed
--
-- This method returns 1 when the status button is shown or 
-- 0 when it is hidden.
-- @param self The given TCalc object.
-- -------------------------------------------------------------------------
   local function IsDisplayed(self)
     return nIsDisplayed
   end

-- -------------------------------------------------------------------------
--- Method Show
--
-- This method shows the calculator.
-- @param self The given TCalc object.
-- -------------------------------------------------------------------------
   local function Show(self)
      -- CalcFrame:Show()
	  CalculatorDialog:Show()
	  CalculatorDialog_TextBox:SetText("")
      nIsDisplayed = 1
   end

-- -------------------------------------------------------------------------
--- Method Toggle
--
-- This method shows/hides the calculator.
-- @param The given TCalc object.
-- -------------------------------------------------------------------------
  local function Toggle(self)
    if (0 == nIsDisplayed) then Show(self) else Hide(self) end
  end

-- -------------------------------------------------------------------------
--- Method OnLoad
--
-- This method registers WoW API events.
-- @param frame The given status button window.
-- -------------------------------------------------------------------------
  local function OnLoad(frame)
    frame:RegisterEvent("PLAYER_ENTERING_WORLD")
    frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    frame:RegisterForDrag("LeftButton")

    if (0 == nIsDisplayed) then Hide(self) else Show(self) end
  end


-- -------------------------------------------------------------------------
--- Method HandleEventPLAYER_ENTERING_WORLD
--
-- This method handles WoW API event "PLAYER_ENTERING_WORLD".
-- -------------------------------------------------------------------------
  local function HandleEventPLAYER_ENTERING_WORLD()
    if semEventPLAYER_ENTERING_WORLD:IsUnLocked() then
      semEventPLAYER_ENTERING_WORLD:Lock()

      tblPower:Toggle()

      semEventPLAYER_ENTERING_WORLD:UnLock()
    end
  end




-- -------------------------------------------------------------------------
--- Method OnEvent
--
-- This method handles WoW API events.
-- @param frame The given status button window.
-- @param event The given WoW event.
-- -------------------------------------------------------------------------
  local function OnEvent(frame, event, ...)
        if event == "PLAYER_ENTERING_WORLD" then HandleEventPLAYER_ENTERING_WORLD()
    end
  end    
  
    
-- -------------------------------------------------------------------------
--- Method GetType
--
-- This method returns the type of this class.
-- -------------------------------------------------------------------------
  local function GetType()
    return "TCalc"
  end

-- -------------------------------------------------------------------------
--- Method new_TCalc
--
-- This method creates an object of this class.
-- -------------------------------------------------------------------------
  function new_TCalc()
    local obj =
    {
        Hide            = Hide,
        IsDisplayed     = IsDisplayed,
        OnEvent         = OnEvent,
        OnLoad          = OnLoad,
        Show            = Show,
        Toggle          = Toggle,
    }

    obj.panel = CreateFrame( "Frame", "MyAddonPanel", UIParent )
    obj.panel.name = "ACalc"

    local strTitle = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge")
    local strOverview = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine2 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine3 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine4 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine5 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine6 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine098 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine099 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine100 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine101 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine102 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine103 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine104 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine105 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine106 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine107 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine108 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strTableHeaderCol1 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormal")
    local strTableHeaderCol2 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormal")
    local strCell11 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell12 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell21 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell22 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell31 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell32 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell41 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell42 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell51 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell52 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell61 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell62 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")

    strTitle:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -15)
    strOverview:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -45)
    strLine2:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -60)
    strLine3:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -75)
    strLine4:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -90)
    strLine5:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -105)
    strLine6:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -120)
    strTableHeaderCol1:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -150)
    strTableHeaderCol2:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 150, -150)
    strCell11:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -165)
    strCell21:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -180)
    strCell31:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -195)
    strCell41:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -210)
    strCell51:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -225)
    strCell61:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -240)
    strCell12:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 150, -165)
    strCell22:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 150, -180)
    strCell32:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 150, -195)
    strCell42:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 150, -210)
    strCell52:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 150, -225)
    strCell62:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 150, -240)
    strLine098:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -260)
    strLine099:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -275)
    strLine100:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -290)
    strLine101:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -305)
    strLine102:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -320)
    strLine103:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -335)
    strLine104:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -350)
    strLine105:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -365)
    strLine106:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -380)
    strLine107:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -395)
    strLine108:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -410)

    strTitle:SetTextColor(0.0, 0.85, 0.0)
    strOverview:SetTextColor(1.0, 1.0, 1.0)
    strLine2:SetTextColor(1.0, 1.0, 1.0)
    strLine3:SetTextColor(1.0, 1.0, 1.0)
    strLine4:SetTextColor(1.0, 1.0, 1.0)
    strLine5:SetTextColor(1.0, 1.0, 1.0)
    strLine6:SetTextColor(1.0, 1.0, 1.0)
    strCell11:SetTextColor(1.0, 1.0, 1.0)
    strCell21:SetTextColor(1.0, 1.0, 1.0)
    strCell31:SetTextColor(1.0, 1.0, 1.0)
    strCell41:SetTextColor(1.0, 1.0, 1.0)
    strCell51:SetTextColor(1.0, 1.0, 1.0)
    strCell61:SetTextColor(1.0, 1.0, 1.0)
    strCell12:SetTextColor(1.0, 1.0, 1.0)
    strCell22:SetTextColor(1.0, 1.0, 1.0)
    strCell32:SetTextColor(1.0, 1.0, 1.0)
    strCell42:SetTextColor(1.0, 1.0, 1.0)
    strCell52:SetTextColor(1.0, 1.0, 1.0)
    strCell62:SetTextColor(1.0, 1.0, 1.0)
    strLine098:SetTextColor(1.0, 1.0, 1.0)
    strLine099:SetTextColor(1.0, 1.0, 1.0)
    strLine100:SetTextColor(1.0, 1.0, 1.0)
    strLine101:SetTextColor(1.0, 1.0, 1.0)
    strLine102:SetTextColor(1.0, 1.0, 1.0)
    strLine103:SetTextColor(1.0, 1.0, 1.0)
    strLine104:SetTextColor(1.0, 1.0, 1.0)
    strLine105:SetTextColor(1.0, 1.0, 1.0)
    strLine106:SetTextColor(1.0, 1.0, 1.0)
    strLine107:SetTextColor(1.0, 1.0, 1.0)
    strLine108:SetTextColor(1.0, 1.0, 1.0)
    strTitle:SetText("ACalc V" ..GetAddOnMetadata("ACalc", "Version" ) .. "   by " .. GetAddOnMetadata("ACalc", "Author"))
    strOverview:SetText("Do you need to add, subtract, multiply, or divide?")
    strLine2:SetText   ("ACalc is an rpn calculator add-on whose primary purpose is to do just that.")

    strTableHeaderCol1:SetText("Slash Command")
    strTableHeaderCol2:SetText("Description")
    strCell31:SetText("/acalc show")
    strCell32:SetText("shows the calculator")             
    strCell41:SetText("/acalc hide")
    strCell42:SetText("hides the calculator")             
    strCell51:SetText("/acalc config")
    strCell52:SetText("display configuration options")             
    strCell61:SetText("/eval <expr>")
    strCell62:SetText("evaluate the given expression")             


    strLine098:SetText("Copyright (C) 2016-2016 Eric T. Hughes")
    strLine099:SetText("")
    strLine100:SetText("This add-on is free software: you can redistribute it and/or modify")
    strLine101:SetText("it under the terms of the GNU General Public License as published by")
    strLine102:SetText("the Free Software Foundation, either version 3 of the License, or")
    strLine103:SetText("(at your option) any later version.")
    strLine104:SetText("")
    strLine105:SetText("This add-on is distributed in the hope that it will be useful,")
    strLine106:SetText("but WITHOUT ANY WARRANTY; without even the implied warranty of")
    strLine107:SetText("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU")
    strLine108:SetText("General Public License for more details. <http://www.gnu.org/licenses/>")

    obj.panel.strTitle = strTitle
    obj.panel.strOverview = strOverview
    obj.panel.strLine2 = strLine2
    obj.panel.strTableHeader = strTableHeader
    obj.panel.strCell11 = strCell11
    obj.panel.strCell21 = strCell21
    obj.panel.strCell31 = strCell31
    obj.panel.strCell41 = strCell41
    obj.panel.strCell51 = strCell51
    obj.panel.strCell61 = strCell61
    obj.panel.strCell12 = strCell12
    obj.panel.strCell22 = strCell22
    obj.panel.strCell32 = strCell32
    obj.panel.strCell42 = strCell42    
    obj.panel.strCell52 = strCell52    
    obj.panel.strCell62 = strCell62    
    obj.panel.strLine098 = strLine098
    obj.panel.strLine099 = strLine099
    obj.panel.strLine100 = strLine100
    obj.panel.strLine101 = strLine101
    obj.panel.strLine102 = strLine102
    obj.panel.strLine103 = strLine103
    obj.panel.strLine104 = strLine104
    obj.panel.strLine105 = strLine105
    obj.panel.strLine106 = strLine106
    obj.panel.strLine107 = strLine107
    obj.panel.strLine108 = strLine108
    obj.panel:Hide()
    InterfaceOptions_AddCategory(obj.panel);
    return obj
  end 
end


tblPower = new_TPower()
tblCalc  = new_TCalc()



-- -------------------------------------------------------------------------
--- Function SlashCmdCalcHandler
--
-- This function does one of the following:
--    - ''       display slash command options
--    - 'config' display configuration options
-- @param val The given value.
-- -------------------------------------------------------------------------
function SlashCmdCalcHandler(val)
  if (nil ~= Broker_ACalc) then
    Broker_ACalc.obj.label = "ACalc"
  end

      if val == "" or val == "?" or val == "help" or val == "HELP" or val == "Help" then
        strMessage = string.format("[ACalc] says: Here are my slash commands:\n%3s\n%3s\n%3s\n%3s", 
              "/acalc show  - shows the calculator",
              "/acalc hide   - hides the calculator", 
              "/acalc config - display configuration options",
			  "/eval <expr> - evaluate the given expression")
        DEFAULT_CHAT_FRAME:AddMessage(strMessage, 0.0, 0.85, 0.0)
		
        if (nil ~= Broker_ACalc) then
          Broker_ACalc.obj.text = "?"
        end
      else
        if val == "config" or val == "CONFIG" or val == "Config" then
          if (nil ~= Broker_ACalc) then
            Broker_ACalc.obj.text = "CONFIG"
          end

          InterfaceOptionsFrame_OpenToCategory(tblCalc.panel)
          InterfaceOptionsFrame_OpenToCategory(tblCalc.panel.name)

        else
          if val == "show" or val == "SHOW" or val == "Show" then
            tblCalc:Show()
            if (nil ~= Broker_ACalc) then
              Broker_ACalc.obj.text = "SHOW"
            end
          else 
            if val == "hide" or val == "HIDE" or val == "Hide" then
              tblCalc:Hide()      
              if (nil ~= Broker_ACalc) then
                Broker_ACalc.obj.text = "HIDE"
              end
			else
              strResult = tblDialogCalculator.tblCalculator.Evaluate(val)

              if "error"==strResult then
                strResult = tblDialogCalculator.tblCalculator.EvaluateInfix(val)
              end
			  
	          if "error"==strResult then
			    -- Note: The 'SimpleCalc' addon handles garbage input.  It is okay to pass the expression along.
	            if (IsAddOnLoaded("SimpleCalc")) then
                  local editbox
                  local strText
	
                  editbox=ChatEdit_ChooseBoxForSend(DEFAULT_CHAT_FRAME) --  Get an editbox
                  ChatEdit_ActivateChat(editbox);--   Show the editbox
				  
				  if (nil==val) then
				    strText = "/simplecalc"
				  else
                    strText = string.format("/simplecalc %s", val)
                  end
				  
                  editbox:SetText(strText) -- Command goes here
                  ChatEdit_OnEnterPressed(editbox);-- Process command and hide (runs ChatEdit_SendText() and ChatEdit_DeactivateChat() respectively)	  
	            elseif (IsAddOnLoaded("BankCalc")) then
                  local strText
	
				  if (nil==val) then
				    strText = "WARNING: try '/BankCalc' at your own risk."
				  else
                    strText = string.format("WARNING: try '/BankCalc %s' at your own risk.", val)
                  end

                  strMessage = string.format("[ACalc] says: %s", strText)
                  DEFAULT_CHAT_FRAME:AddMessage(strMessage, 1.00, 0.96, 0.41) -- Light yellow				  
	            elseif (IsAddOnLoaded("Calc")) then
                  local strText
	
				  if (nil==val) then
				    strText = "WARNING: try '/Calc' at your own risk."
				  else
                    strText = string.format("WARNING: try '/Calc %s' at your own risk.", val)
                  end

                  strMessage = string.format("[ACalc] says: %s", strText)
                  DEFAULT_CHAT_FRAME:AddMessage(strMessage, 1.00, 0.96, 0.41) -- Light yellow				  
	            elseif (IsAddOnLoaded("ConsoleCalc")) then
                  local strText
	
				  if (nil==val) then
				    strText = "WARNING: try '/ca' at your own risk."
				  else
                    strText = string.format("WARNING: try '/ca %s' at your own risk.", val)
                  end

                  strMessage = string.format("[ACalc] says: %s", strText)
                  DEFAULT_CHAT_FRAME:AddMessage(strMessage, 1.00, 0.96, 0.41) -- Light yellow				  
	            elseif (IsAddOnLoaded("expr")) then
                  local strText
	
				  if (nil==val) then
				    strText = "WARNING: try '/x' at your own risk."
				  else
                    strText = string.format("WARNING: try '/x %s' at your own risk.", val)
                  end

                  strMessage = string.format("[ACalc] says: %s", strText)
                  DEFAULT_CHAT_FRAME:AddMessage(strMessage, 1.00, 0.96, 0.41) -- Light yellow				  
				-- Note: The 'xcalc' addon does not handle garbage input.  It is not okay to pass the expression along.
	            elseif (IsAddOnLoaded("xcalc")) then
                  local strText
	
				  if (nil==val) then
				    strText = "WARNING: try '/xcalc' at your own risk."
				  else
                    strText = string.format("WARNING: try '/xcalc %s' at your own risk.", val)
                  end

                  strMessage = string.format("[ACalc] says: %s", strText)
                  DEFAULT_CHAT_FRAME:AddMessage(strMessage, 1.00, 0.96, 0.41) -- Light yellow				  
				else
                  strMessage = string.format("[ACalc] says: %s = %s", strResult, val)
                  DEFAULT_CHAT_FRAME:AddMessage(strMessage, 1.00, 0.42, 0.42) -- Light Red
                  strMessage = string.format("[ACalc] says: Here are my slash commands:\n%3s\n%3s\n%3s\n%3s", 
                       "/acalc show  - shows the calculator",
                       "/acalc hide   - hides the calculator", 
                       "/acalc config - display configuration options",
			           "/eval <expr> - evaluate the given expression")
                  DEFAULT_CHAT_FRAME:AddMessage(strMessage, 0.0, 0.85, 0.0) -- Green
		
                  if (nil ~= Broker_ACalc) then
                    Broker_ACalc.obj.text = "?"
                  end
	            end
	          else			  
                strMessage = string.format("[ACalc] says: %s = %s", strResult, val)
                DEFAULT_CHAT_FRAME:AddMessage(strMessage, 0.0, 0.85, 0.0)
	          end			  
            end
          end
        end
      end
end 

-- -------------------------------------------------------------------------
--- Function SlashCmdEvalHandler
--
-- This function evaluates a given expression.
-- @param a_strExpression The given expression.
-- -------------------------------------------------------------------------
function SlashCmdEvalHandler(a_strExpression)
  if (nil ~= Broker_ACalc) then
    Broker_ACalc.obj.label = "ACalc"
  end

  if a_strExpression == "" or a_strExpression == "?" or a_strExpression == "help" or a_strExpression == "HELP" or a_strExpression == "Help" then
    strMessage = string.format("[ACalc] says: Here are my slash commands:\n%3s\n%3s\n%3s\n%3s", 
          "/acalc show  - shows the calculator",
          "/acalc hide   - hides the calculator", 
          "/acalc config - display configuration options",
		  "/eval <expr> - evaluate the given expression")
    DEFAULT_CHAT_FRAME:AddMessage(strMessage, 0.0, 0.85, 0.0)
		
    if (nil ~= Broker_ACalc) then
      Broker_ACalc.obj.text = "?"
    end
  else
    strResult = tblDialogCalculator.tblCalculator.Evaluate(a_strExpression)    

    if "error"==strResult then
      strResult = tblDialogCalculator.tblCalculator.EvaluateInfix(a_strExpression)
    end

	if "error"==strResult then
      strMessage = string.format("[ACalc] says: %s = %s", strResult, a_strExpression)
      DEFAULT_CHAT_FRAME:AddMessage(strMessage, 1.00, 0.42, 0.42) -- Light Red
      strMessage = string.format("[ACalc] says: Here are my slash commands:\n%3s\n%3s\n%3s\n%3s", 
        "/acalc show  - shows the calculator",
        "/acalc hide   - hides the calculator", 
        "/acalc config - display configuration options",
	    "/eval <expr> - evaluate the given expression")
      DEFAULT_CHAT_FRAME:AddMessage(strMessage, 0.0, 0.85, 0.0) -- Green	
	else
      strMessage = string.format("[ACalc] says: %s = %s", strResult, a_strExpression)
      DEFAULT_CHAT_FRAME:AddMessage(strMessage, 0.0, 0.85, 0.0)
	end	
  end
end 



SLASH_ACALC1, SLASH_ACALC2, SLASH_ACALC3, SLASH_ACALC4  = "/acalc", "/acalc show", "/acalc hide", "/acalc config"
SLASH_EVAL1, SLASH_EVAL2, SLASH_EVAL3, SLASH_EVAL4 = "/eval", "/Eval", "/EVAL"
SlashCmdList["ACALC"] = SlashCmdCalcHandler
SlashCmdList["EVAL"] = SlashCmdEvalHandler