  -- =========================================================================
  -- Class TStack
  --
  -- This class defines a stack.
  -- =========================================================================
  do
  -- -------------------------------------------------------------------------
  -- Method IsEmpty
  --
  -- This method returns true when the stack is empty or false otherwise.
  -- -------------------------------------------------------------------------
    local function IsEmpty(self)
      return (#self.m_vecItems == 0)
    end

  -- -------------------------------------------------------------------------
  -- Method Peek
  --
  -- This method returns the most recently stacked item without popping it off
  -- -------------------------------------------------------------------------
    local function Peek(self)
      local Item 
	  
      if (0 < #self.m_vecItems) then
        Item = self.m_vecItems[#self.m_vecItems]
      end
	  
      return Item
    end  
  
  -- -------------------------------------------------------------------------
  -- Method Pop
  --
  -- This method returns the most recently stacked item.
  -- -------------------------------------------------------------------------
    local function Pop(self)
      local Item 
	  
      if (0 < #self.m_vecItems) then
        Item = self.m_vecItems[#self.m_vecItems]
        table.remove(self.m_vecItems)
      end
	  
      return Item
    end  

  -- -------------------------------------------------------------------------
  -- Method Push
  --
  -- This method places an item on the stack.
  -- -------------------------------------------------------------------------
    local function Push(self, a_Item)
      table.insert(self.m_vecItems, a_Item)   
    end

  -- -------------------------------------------------------------------------
  -- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
     local function GetType()
       return "TStack"
     end

  -- -------------------------------------------------------------------------
  -- Method Size
  --
  -- This method returns the number of items on the stack.
  -- -------------------------------------------------------------------------
    local function Size(self)
      local numItemCount
	  
	  numItemCount = #self.m_vecItems
	  
      return numItemCount
    end  
	 
  -- -------------------------------------------------------------------------
  -- Method new_TStack
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TStack()
      local obj =
      {
        GetType    = GetType,
        IsEmpty    = IsEmpty,
		Peek       = Peek,
        Pop        = Pop,
        Push       = Push,
		Size       = Size,
        m_vecItems = {},
      }
	  
      return obj
    end
  end


do -- Class TModelCalculator
  -- -------------------------------------------------------------------------
  -- Method SetValue
  --
  -- This method pushes a value onto the stack for later evaluation.
  -- -------------------------------------------------------------------------
  local function SetValue(self)
    local strValue
    local numValue

    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)
	numValue = tonumber(strValue)
    self.tblStack:Push(numValue)
	self.strValue = ""
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDecimalPoint
  --
  -- This method appends a decimal point to the string when there is not 
  -- already a decimal point in the string and it would not be the 15th
  -- character in the string.
  -- -------------------------------------------------------------------------
  local function AppendDecimalPoint(self)
    local strValue
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)

	-- Note: '.' is a wildcard character.  
	--       It should be escaped with a % to actually search for a decimal point.
	if (nil==string.find(strValue, '%.')) then
	  local numStringLength
	  
	  numStringLength = string.len(strValue)
	  
	  if (15 > numStringLength) then
	    if (0 == numStringLength) then
		  strValue = "0."
		else
	      strValue = string.format("%s.", strValue)
		end
	  end
    end
	
	self.strValue = strValue
    return self.strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit0
  --
  -- This method appends a zero to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------
  local function AppendDigit0(self)
    local strValue
    local numStringLength	
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)
	numStringLength = string.len(strValue)
	
	if (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s0", strValue)
	  end
    end
	
	self.strValue = strValue
    return self.strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit1
  --
  -- This method appends a one to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------
  local function AppendDigit1(self)
    local strValue 
    local numStringLength
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numStringLength = string.len(strValue)
	
	if (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s1", strValue)
	  else
	    strValue = "1"
	  end
    end
	
	self.strValue = strValue
    return self.strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit2
  --
  -- This method appends a two to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------
  local function AppendDigit2(self)
    local strValue 
    local numStringLength
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numStringLength = string.len(strValue)
	
	if (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s2", strValue)
	  else
	    strValue = "2"
	  end
    end
	
	self.strValue = strValue
    return self.strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit3
  --
  -- This method appends a three to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------
  local function AppendDigit3(self)
    local strValue 
    local numStringLength
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numStringLength = string.len(strValue)
	
	if (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s3", strValue)
	  else
	    strValue = "3"
	  end
    end
	
	self.strValue = strValue
    return self.strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit4
  --
  -- This method appends a four to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------
  local function AppendDigit4(self)
    local strValue 
    local numStringLength
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numStringLength = string.len(strValue)
	
	if (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s4", strValue)
	  else
	    strValue = "4"
	  end
    end
	
	self.strValue = strValue
    return self.strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit5
  --
  -- This method appends a five to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------  
  local function AppendDigit5(self)
    local strValue 
    local numStringLength
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numStringLength = string.len(strValue)
	
	if (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s5", strValue)
	  else
	    strValue = "5"
	  end
    end
	
	self.strValue = strValue
    return self.strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit6
  --
  -- This method appends a six to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------  
  local function AppendDigit6(self)
    local strValue 
    local numStringLength
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numStringLength = string.len(strValue)
	
	if (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s6", strValue)
	  else
	    strValue = "6"
	  end
    end
	
	self.strValue = strValue
    return self.strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit7
  --
  -- This method appends a seven to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------  
  local function AppendDigit7(self)
    local strValue 
    local numStringLength
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numStringLength = string.len(strValue)
	
	if (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s7", strValue)
	  else
	    strValue = "7"
	  end
    end
	
	self.strValue = strValue
    return self.strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit8
  --
  -- This method appends an eight to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------
  local function AppendDigit8(self)
    local strValue 
    local numStringLength
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numStringLength = string.len(strValue)
	
	if (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s8", strValue)
	  else
	    strValue = "8"
	  end
    end
	
	self.strValue = strValue
    return self.strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit9
  --
  -- This method appends a two to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------
  local function AppendDigit9(self)
    local strValue 
    local numStringLength
	local numStringLengthMax
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numStringLength = string.len(strValue)
	
	if (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s9", strValue)
	  else
	    strValue = "9"
	  end
    end
	
	self.strValue = strValue
    return self.strValue
  end

  -- -------------------------------------------------------------------------
  -- Method ToggleSign
  --
  -- This method toggles the sign of the number.
  -- -------------------------------------------------------------------------
  local function ToggleSign(self)
    local strValue
	local strValueToggled
    local numStringLength
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numStringLength = string.len(strValue)
	
	if (0 < numStringLength) then 
	  if (1==string.find(strValue, '-')) then
	    strValueToggled = string.sub(strValue, 2, numStringLength)
	  else
	    strValueToggled = string.format("-%s", strValue)
	  end
	else
	  strValueToggled = "-"
	end
		
	self.strValue = strValueToggled
    return self.strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method GetPlusSign
  --
  -- This method returns the plus sign.
  -- -------------------------------------------------------------------------
  local function GetPlusSign()
    return "+"
  end
  
  -- -------------------------------------------------------------------------
  -- Method GetMinusSign
  --
  -- This method returns the minus sign.
  -- -------------------------------------------------------------------------
  local function GetMinusSign()
    return "-"
  end
  
  -- -------------------------------------------------------------------------
  -- Method GetMultiplySign
  --
  -- This method returns the sign for multiplication.
  -- -------------------------------------------------------------------------
  local function GetMultiplySign()
    return "*"
  end
  
  -- -------------------------------------------------------------------------
  -- Method GetDivideSign
  --
  -- This method returns the sign for division.
  -- -------------------------------------------------------------------------
  local function GetDivideSign()
    return "/"
  end

  -- -------------------------------------------------------------------------
  -- Method GetPowerSign
  --
  -- This method returns the sign for power.
  -- -------------------------------------------------------------------------
  local function GetPowerSign()
    return "^"
  end
  
  -- -------------------------------------------------------------------------
  -- Method GetPlusMinus
  --
  -- This method returns the sign for plus or minus.
  -- -------------------------------------------------------------------------
  local function GetPlusMinus()
    return "±"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForAbsoluteValue
  --
  -- This method returns the text for absolute value.
  -- -------------------------------------------------------------------------
  local function GetTextForAbsoluteValue()
    return "math.abs"
  end
  
  -- -------------------------------------------------------------------------
  -- Method GetTextForArcCosine
  --
  -- This method returns the text for arccosine.
  -- -------------------------------------------------------------------------
  local function GetTextForArcCosine()
    return "math.acos"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForArcSine
  --
  -- This method returns the text for arccosine.
  -- -------------------------------------------------------------------------
  local function GetTextForArcSine()
    return "math.asin"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForArcTangent
  --
  -- This method returns the text for arctangent
  -- -------------------------------------------------------------------------
  local function GetTextForArcTangent()
    return "math.atan"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForArcTangent2
  --
  -- This method returns the text for arctangent2
  -- -------------------------------------------------------------------------
  local function GetTextForArcTangent2()
    return "math.atan2"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForCeiling
  --
  -- This method returns the text for ceiling.
  -- -------------------------------------------------------------------------
  local function GetTextForCeiling()
    return "math.ceil"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForCosine
  --
  -- This method returns the text for cosine.
  -- -------------------------------------------------------------------------
  local function GetTextForCosine()
    return "math.cos"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForHyperbolicCosine
  --
  -- This method returns the text for hyperbolic cosine.
  -- -------------------------------------------------------------------------
  local function GetTextForHyperbolicCosine()
    return "math.cosh"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForDegrees
  --
  -- This method returns the text for degrees.
  -- -------------------------------------------------------------------------
  local function GetTextForDegrees()
    return "math.deg"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForExp
  --
  -- This method returns the text for e.
  -- -------------------------------------------------------------------------
  local function GetTextForExp()
    return "math.exp"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForFloor
  --
  -- This method returns the text for floor.
  -- -------------------------------------------------------------------------
  local function GetTextForFloor()
    return "math.floor"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForFMod
  --
  -- This method returns the text for fmod.
  -- -------------------------------------------------------------------------
  local function GetTextForFMod()
    return "math.fmod"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForNaturalLogarithm
  --
  -- This method returns the text for natural logarithm.
  -- -------------------------------------------------------------------------
  local function GetTextForNaturalLogarithm()
    return "math.log"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForBase10Logarithm
  --
  -- This method returns the text for base 10 logarithm.
  -- -------------------------------------------------------------------------
  local function GetTextForBase10Logarithm()
    return "math.log10"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForMaximum
  --
  -- This method returns the text for maximum.
  -- -------------------------------------------------------------------------
  local function GetTextForMaximum()
    return "math.max"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForMinimum
  --
  -- This method returns the text for minimum.
  -- -------------------------------------------------------------------------
  local function GetTextForMinimum()
    return "math.min"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForPi
  --
  -- This method returns the text for pi.
  -- -------------------------------------------------------------------------
  local function GetTextForPi()
    return "math.pi"
  end
  
  -- -------------------------------------------------------------------------
  -- Method GetTextForPower
  --
  -- This method returns the text for power.
  -- -------------------------------------------------------------------------
  local function GetTextForPower()
    return "math.pow"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForRadians
  --
  -- This method returns the text for radians.
  -- -------------------------------------------------------------------------
  local function GetTextForRadians()
    return "math.rad"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForSine
  --
  -- This method returns the text for sine.
  -- -------------------------------------------------------------------------
  local function GetTextForSine()
    return "math.sin"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForHyperbolicSine
  --
  -- This method returns the text for hyperbolic sine.
  -- -------------------------------------------------------------------------
  local function GetTextForHyperbolicSine()
    return "math.sinh"
  end
  
  -- -------------------------------------------------------------------------
  -- Method GetTextForSquareRoot
  --
  -- This method returns the text for square root.
  -- -------------------------------------------------------------------------
  local function GetTextForSquareRoot()
    return "math.sqrt"
  end
  
  -- -------------------------------------------------------------------------
  -- Method GetTextForTangent
  --
  -- This method returns the text for tangent.
  -- -------------------------------------------------------------------------
  local function GetTextForTangent()
    return "math.tan"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForHyperbolicTangent
  --
  -- This method returns the text for hyperbolic tangent.
  -- -------------------------------------------------------------------------
  local function GetTextForHyperbolicTangent()
    return "math.tanh"
  end
  
  -- -------------------------------------------------------------------------
  -- Method Add
  --
  -- This method adds two numbers when there are two numbers to add.
  -- -------------------------------------------------------------------------
  local function Add(self)
    local numValue1
	local numValue2
	local numReturnValue
	local numStackSize
	
	self:SetValue()
	numStackSize = self.tblStack:Size()
	
	if (1 < numStackSize) then
	  numValue2 = self.tblStack:Pop()
	  numValue1 = self.tblStack:Pop()
	  numReturnValue = numValue1 + numValue2
	  self.tblStack:Push(numReturnValue)
	end
	
	self.strValue = ""
	return numReturnValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method Subtract
  --
  -- This method subtracts two numbers when there are two numbers to subtract.
  -- -------------------------------------------------------------------------
  local function Subtract(self)
    local numValue1
	local numValue2
	local numReturnValue
	local numStackSize
	
	self:SetValue()
	numStackSize = self.tblStack:Size()

	if (1 < numStackSize) then
	  numValue2 = self.tblStack:Pop()
	  numValue1 = self.tblStack:Pop()
	  numReturnValue = numValue1 - numValue2
	  self.tblStack:Push(numReturnValue)
	end
	
	self.strValue = ""
	return numReturnValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method Multiply
  --
  -- This method multiplies two numbers when there are two numbers to multiply.
  -- -------------------------------------------------------------------------
  local function Multiply(self)
    local numValue1
	local numValue2
	local numReturnValue
	local numStackSize
	
	self:SetValue()
	numStackSize = self.tblStack:Size()

	if (1 < numStackSize) then
      numValue2 = self.tblStack:Pop()
      numValue1 = self.tblStack:Pop()
      numReturnValue = numValue1 * numValue2
	  self.tblStack:Push(numReturnValue)
    end
	
	self.strValue = ""
	return numReturnValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method Divide
  --
  -- This method divides two numbers when there are two numbers to divide and
  -- when it is defined.
  -- -------------------------------------------------------------------------  
  local function Divide(self)
    local numValue1
	local numValue2
	local numReturnValue
	
	self:SetValue()
	numStackSize = self.tblStack:Size()

	if (1 < numStackSize) then
      numValue2 = self.tblStack:Pop()
	  
	  if (0 == numValue2) then -- protect against division by zero
	    self.tblStack:Push(numValue2)
      else
        numValue1 = self.tblStack:Pop()
        numReturnValue = numValue1 / numValue2
	    self.tblStack:Push(numReturnValue)
      end
	end
	
	self.strValue = ""
	return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method Power
  --
  -- This method raise a number to a power.
  -- -------------------------------------------------------------------------  
  local function Power(self)
    local numValue1
	local numValue2
	local numReturnValue
	
	self:SetValue()
	numStackSize = self.tblStack:Size()

	if (1 < numStackSize) then
      numValue2 = self.tblStack:Pop()	  
      numValue1 = self.tblStack:Pop()
      numReturnValue = math.pow(numValue1, numValue2)
      self.tblStack:Push(numReturnValue)
	end
	
	self.strValue = ""
	return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method AbsoluteValue
  --
  -- This method returns the absolute value of a given value.
  -- -------------------------------------------------------------------------  
  local function AbsoluteValue(self)
    local numValue1
	local numReturnValue
	
	self:SetValue()
	numStackSize = self.tblStack:Size()

	if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
      numReturnValue = math.abs(numValue1)
      self.tblStack:Push(numReturnValue)
	end
	
	self.strValue = ""
	return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method ArcCosine
  --
  -- This method returns the arccosine of a given value.
  -- -------------------------------------------------------------------------  
  local function ArcCosine(self)
    local numValue1
	local numReturnValue
	
	self:SetValue()
	numStackSize = self.tblStack:Size()

	if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
      numReturnValue = math.acos(numValue1)
      self.tblStack:Push(numReturnValue)
	end
	
	self.strValue = ""
	return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method ArcSine
  --
  -- This method returns the arcsine of a given value.
  -- -------------------------------------------------------------------------  
  local function ArcSine(self)
    local numValue1
	local numReturnValue
	
	self:SetValue()
	numStackSize = self.tblStack:Size()

	if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
      numReturnValue = math.asin(numValue1)
      self.tblStack:Push(numReturnValue)
	end
	
	self.strValue = ""
	return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method ArcTangent
  --
  -- This method returns the arctangent of a given value.
  -- -------------------------------------------------------------------------  
  local function ArcTangent(self)
    local numValue1
	local numReturnValue
	
	self:SetValue()
	numStackSize = self.tblStack:Size()
	
	if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
      numReturnValue = math.atan(numValue1)
      self.tblStack:Push(numReturnValue)
	end
	
	self.strValue = ""
	return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method ArcTangent2
  --
  -- This method returns the arctangent of a given value.
  -- -------------------------------------------------------------------------  
  local function ArcTangent2(self)
    local numValue1
	local numValue2
	local numReturnValue
	
	self:SetValue()
	numStackSize = self.tblStack:Size()
	
	if (1 < numStackSize) then
      numValue2 = self.tblStack:Pop()
	  numValue1 = self.tblStack:Pop()
      numReturnValue = math.atan2(numValue1, numValue2)
      self.tblStack:Push(numReturnValue)
	end
	
	self.strValue = ""
	return numReturnValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method Ceiling
  --
  -- This method returns the ceiling.
  -- -------------------------------------------------------------------------  
  local function Ceiling(self)
    local numValue1
	local numReturnValue
	
	self:SetValue()
	numStackSize = self.tblStack:Size()
	
	if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
      numReturnValue = math.ceil(numValue1)
      self.tblStack:Push(numReturnValue)
	end
	
	self.strValue = ""
	return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method Cosine
  --
  -- This method returns the cosine.
  -- -------------------------------------------------------------------------  
  local function Cosine(self)
    local numValue1
	local numReturnValue
	
	self:SetValue()
	numStackSize = self.tblStack:Size()
	
	if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
      numReturnValue = math.cos(numValue1)
      self.tblStack:Push(numReturnValue)
	end
	
	self.strValue = ""
	return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method HyperbolicCosine
  --
  -- This method returns the hyperbolic cosine.
  -- -------------------------------------------------------------------------  
  local function HyperbolicCosine(self)
    local numValue1
	local numReturnValue
	
	self:SetValue()
	numStackSize = self.tblStack:Size()
	
	if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
      numReturnValue = math.cosh(numValue1)
      self.tblStack:Push(numReturnValue)
	end
	
	self.strValue = ""
	return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method Degrees
  --
  -- This method returns the hyperbolic cosine.
  -- -------------------------------------------------------------------------  
  local function Degrees(self)
    local numValue1
	local numReturnValue
	
	self:SetValue()
	numStackSize = self.tblStack:Size()
	
	if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
      numReturnValue = math.deg(numValue1)
      self.tblStack:Push(numReturnValue)
	end
	
	self.strValue = ""
	return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method Exp
  --
  -- This method returns the e to the given power.
  -- -------------------------------------------------------------------------  
  local function Exp(self)
    local numValue1
	local numReturnValue
	
	self:SetValue()
	numStackSize = self.tblStack:Size()
	
	if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
      numReturnValue = math.exp(numValue1)
      self.tblStack:Push(numReturnValue)
	end
	
	self.strValue = ""
	return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method Floor
  --
  -- This method returns the floor of a given value.
  -- -------------------------------------------------------------------------  
  local function Floor(self)
    local numValue1
	local numReturnValue
	
	self:SetValue()
	numStackSize = self.tblStack:Size()
	
	if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
      numReturnValue = math.floor(numValue1)
      self.tblStack:Push(numReturnValue)
	end
	
	self.strValue = ""
	return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method FMod
  --
  -- This method returns the fmod of a given value.
  -- -------------------------------------------------------------------------  
  local function FMod(self)
    local numValue1
	local numValue2
	local numReturnValue
	
	self:SetValue()
	numStackSize = self.tblStack:Size()
	
	if (1 < numStackSize) then
	  numValue2 = self.tblStack:Pop()
      numValue1 = self.tblStack:Pop()
      numReturnValue = math.fmod(numValue1, numValue2)
      self.tblStack:Push(numReturnValue)
	end
	
	self.strValue = ""
	return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method NaturalLogarithm
  --
  -- This method returns the natural logarithm
  -- -------------------------------------------------------------------------  
  local function NaturalLogarithm(self)
    local numValue1
	local numReturnValue
	
	self:SetValue()
	numStackSize = self.tblStack:Size()
	
	if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
      numReturnValue = math.log(numValue1)
      self.tblStack:Push(numReturnValue)
	end
	
	self.strValue = ""
	return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method Base10Logarithm
  --
  -- This method returns the base 10 logarithm
  -- -------------------------------------------------------------------------  
  local function Base10Logarithm(self)
    local numValue1
	local numReturnValue
	
	self:SetValue()
	numStackSize = self.tblStack:Size()
	
	if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
      numReturnValue = math.log10(numValue1)
      self.tblStack:Push(numReturnValue)
	end
	
	self.strValue = ""
	return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method Maximum
  --
  -- This method returns the maximum
  -- -------------------------------------------------------------------------  
  local function Maximum(self)
    local numValue1
	local numValue2
	local numReturnValue
	
	self:SetValue()
	numStackSize = self.tblStack:Size()
	
	if (1 < numStackSize) then
	  numValue2 = self.tblStack:Pop()
      numValue1 = self.tblStack:Pop()
      numReturnValue = math.max(numValue1, numValue2)
      self.tblStack:Push(numReturnValue)
	end
	
	self.strValue = ""
	return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method Minimum
  --
  -- This method returns the minimum
  -- -------------------------------------------------------------------------  
  local function Minimum(self)
    local numValue1
	local numValue2
	local numReturnValue
	
	self:SetValue()
	numStackSize = self.tblStack:Size()
	
	if (1 < numStackSize) then
	  numValue2 = self.tblStack:Pop()
      numValue1 = self.tblStack:Pop()
      numReturnValue = math.min(numValue1, numValue2)
      self.tblStack:Push(numReturnValue)
	end
	
	self.strValue = ""
	return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method Pi
  --
  -- This method returns pi
  -- -------------------------------------------------------------------------  
  local function Pi(self)
	local numReturnValue
	
    numReturnValue = math.pi	
    self.tblStack:Push(numReturnValue)
	self.strValue = ""
	return numReturnValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method Radians
  --
  -- This method returns the radians
  -- -------------------------------------------------------------------------  
  local function Radians(self)
    local numValue1
	local numReturnValue
	
	self:SetValue()
	numStackSize = self.tblStack:Size()
	
	if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
      numReturnValue = math.rad(numValue1)
      self.tblStack:Push(numReturnValue)
	end
	
	self.strValue = ""
	return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method Sine
  --
  -- This method returns the sine
  -- -------------------------------------------------------------------------  
  local function Sine(self)
    local numValue1
	local numReturnValue
	
	self:SetValue()
	numStackSize = self.tblStack:Size()
	
	if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
      numReturnValue = math.sin(numValue1)
      self.tblStack:Push(numReturnValue)
	end
	
	self.strValue = ""
	return numReturnValue
  end

    -- -------------------------------------------------------------------------
  -- Method HyperbolicSine
  --
  -- This method returns the hyperbolic sine
  -- -------------------------------------------------------------------------  
  local function HyperbolicSine(self)
    local numValue1
	local numReturnValue
	
	self:SetValue()
	numStackSize = self.tblStack:Size()
	
	if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
      numReturnValue = math.sinh(numValue1)
      self.tblStack:Push(numReturnValue)
	end
	
	self.strValue = ""
	return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method SquareRoot
  --
  -- This method returns the square root
  -- -------------------------------------------------------------------------  
  local function SquareRoot(self)
    local numValue1
	local numReturnValue
	
	self:SetValue()
	numStackSize = self.tblStack:Size()
	
	if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
      numReturnValue = math.sqrt(numValue1)
      self.tblStack:Push(numReturnValue)
	end
	
	self.strValue = ""
	return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method Tangent
  --
  -- This method returns the tangent
  -- -------------------------------------------------------------------------  
  local function Tangent(self)
    local numValue1
	local numReturnValue
	
	self:SetValue()
	numStackSize = self.tblStack:Size()
	
	if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
      numReturnValue = math.tan(numValue1)
      self.tblStack:Push(numReturnValue)
	end
	
	self.strValue = ""
	return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method HyperbolicTangent
  --
  -- This method returns the hyperbolic tangent
  -- -------------------------------------------------------------------------  
  local function HyperbolicTangent(self)
    local numValue1
	local numReturnValue
	
	self:SetValue()
	numStackSize = self.tblStack:Size()
	
	if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
      numReturnValue = math.tanh(numValue1)
      self.tblStack:Push(numReturnValue)
	end
	
	self.strValue = ""
	return numReturnValue
  end
  
  
  -- -------------------------------------------------------------------------
  -- Method Reset
  --
  -- This method resets the calculator to its initial state.
  -- -------------------------------------------------------------------------
  local function Reset(self)
    -- print("+Reset")
    local numItemCount
	local numValue
	
    self.strValue = ""
	
	numItemCount = self.tblStack:Size()
	
	if (0 < numItemCount) then
      for numIndex = 1, numItemCount, 1
      do
        numValue = self.tblStack:Pop()
		-- print(numValue)
      end
    end
	-- print("-Reset")
  end

  -- -------------------------------------------------------------------------
  -- Method Evaluate
  --
  -- This method evaluates a postfix mathematical expression.
  -- -------------------------------------------------------------------------
local function Evaluate(a_strExpression)
 tblStack = {}  
 z = 0
 strText = ""
 
  for strToken in string.gmatch(a_strExpression, '%S+') do
    if string.find(strToken, '^[-+*/]$')  then
      if 2>#tblStack then z = nil break end
      y = table.remove(tblStack)
	  x = table.remove(tblStack)
      loadstring('z=x'..strToken..'y')()
    elseif string.find(strToken, "%^") then
      if 2>#tblStack then z = nil break end
      y = table.remove(tblStack)
	  x = table.remove(tblStack)
	  z = math.pow(x, y)
	elseif string.find(strToken, "math.pow") then
      if 2>#tblStack then z = nil break end
      y = table.remove(tblStack)
	  x = table.remove(tblStack)
	  z = math.pow(x, y)
    elseif string.find(strToken, "math.abs") then
	  if 1>#tblStack then z = nil break end
	  x = table.remove(tblStack)
	  z = math.abs(x)
    elseif string.find(strToken, "math.acos") then
	  if 1>#tblStack then z = nil break end
	  x = table.remove(tblStack)
	  z = math.acos(x)
    elseif string.find(strToken, "math.asin") then
	  if 1>#tblStack then z = nil break end
	  x = table.remove(tblStack)
	  z = math.asin(x)
    elseif string.find(strToken, "math.atan2") then
	  if 2>#tblStack then z = nil break end
	  y = table.remove(tblStack)
	  x = table.remove(tblStack)
	  z = math.atan2(x,y)
    elseif string.find(strToken, "math.atan") then
	  if 1>#tblStack then z = nil break end
	  x = table.remove(tblStack)
	  z = math.atan(x)
    elseif string.find(strToken, "math.ceil") then
	  if 1>#tblStack then z = nil break end
	  x = table.remove(tblStack)
	  z = math.ceil(x)
    elseif string.find(strToken, "math.cosh") then
	  if 1>#tblStack then z = nil break end
	  x = table.remove(tblStack)
	  z = math.cosh(x)
    elseif string.find(strToken, "math.cos") then
	  if 1>#tblStack then z = nil break end
	  x = table.remove(tblStack)
	  z = math.cos(x)
    elseif string.find(strToken, "math.deg") then
	  if 1>#tblStack then z = nil break end
	  x = table.remove(tblStack)
	  z = math.deg(x)
    elseif string.find(strToken, "math.exp") then
	  if 1>#tblStack then z = nil break end
	  x = table.remove(tblStack)
	  z = math.exp(x)
    elseif string.find(strToken, "math.floor") then
	  if 1>#tblStack then z = nil break end
	  x = table.remove(tblStack)
	  z = math.floor(x)
    elseif string.find(strToken, "math.fmod") then
	  if 2>#tblStack then z = nil break end
	  y = table.remove(tblStack)
	  x = table.remove(tblStack)
	  z = math.fmod(x,y)
    elseif string.find(strToken, "math.log10") then
	  if 1>#tblStack then z = nil break end
	  x = table.remove(tblStack)
	  z = math.log10(x)
    elseif string.find(strToken, "math.log") then
	  if 1>#tblStack then z = nil break end
	  x = table.remove(tblStack)
	  z = math.log(x)
    elseif string.find(strToken, "math.max") then
	  if 2>#tblStack then z = nil break end
	  y = table.remove(tblStack)
	  x = table.remove(tblStack)
	  z = math.max(x, y)
    elseif string.find(strToken, "math.min") then
	  if 2>#tblStack then z = nil break end
	  y = table.remove(tblStack)
	  x = table.remove(tblStack)
	  z = math.min(x, y)
    elseif string.find(strToken, "math.pi") then
	  z = math.pi
    elseif string.find(strToken, "math.rad") then
	  if 1>#tblStack then z = nil break end
	  x = table.remove(tblStack)
	  z = math.rad(x)
    elseif string.find(strToken, "math.sinh") then
	  if 1>#tblStack then z = nil break end
	  x = table.remove(tblStack)
	  z = math.sinh(x)
    elseif string.find(strToken, "math.sin") then
	  if 1>#tblStack then z = nil break end
	  x = table.remove(tblStack)
	  z = math.sin(x)
    elseif string.find(strToken, "math.sqrt") then
	  if 1>#tblStack then z = nil break end
	  x = table.remove(tblStack)
	  z = math.sqrt(x)
    elseif string.find(strToken, "math.tanh") then
	  if 1>#tblStack then z = nil break end
	  x = table.remove(tblStack)
	  z = math.tanh(x)
    elseif string.find(strToken, "math.tan") then
	  if 1>#tblStack then z = nil break end
	  x = table.remove(tblStack)
	  z = math.tan(x)
	else
      z = tonumber(strToken)  if nil==z then break end
    end
    table.insert(tblStack, z)
  end
  
  numItemCount = #tblStack
  
  if 1==numItemCount and z then 
    strText = string.format("%s", z)
  elseif 1<numItemCount or nil==z then 
    strText = "error"
  end
  
  return strText
end

  -- -------------------------------------------------------------------------
  -- Method EvaluateInfix
  --
  -- This method evaluates an infix expression in Lua syntax.
  -- -------------------------------------------------------------------------
  local function EvaluateInfix(a_strExpression)
    local fn
    local strText = ""
  
    if pcall(loadstring("return " .. a_strExpression)) then
      fn = loadstring("return " .. a_strExpression)
  
      if not fn or not fn() then
        strText = "error"
      else
        strText = string.format("%s", fn())
      end
	else
	  strText = "error"
	end

    return strText
  end
    
  -- -------------------------------------------------------------------------
  -- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
  local function GetType()
    return "TModelCalculator"
  end

    local objGetTexts =
	{
	  GetTextForAbsoluteValue     = GetTextForAbsoluteValue,
	  GetTextForArcCosine         = GetTextForArcCosine,
	  GetTextForArcSine           = GetTextForArcSine,
	  GetTextForArcTangent        = GetTextForArcTangent,
	  GetTextForArcTangent2       = GetTextForArcTangent2,
	  GetTextForCeiling           = GetTextForCeiling,
	  GetTextForCosine            = GetTextForCosine,
	  GetTextForHyperbolicCosine  = GetTextForHyperbolicCosine,
	  GetTextForDegrees           = GetTextForDegrees,
	  GetTextForExp               = GetTextForExp,
	  GetTextForFloor             = GetTextForFloor,
	  GetTextForFMod              = GetTextForFMod,
	  GetTextForNaturalLogarithm  = GetTextForNaturalLogarithm,
	  GetTextForBase10Logarithm   = GetTextForBase10Logarithm,
	  GetTextForMaximum           = GetTextForMaximum,
	  GetTextForMinimum           = GetTextForMinimum,
	  GetTextForPi                = GetTextForPi,
	  GetTextForPower             = GetTextForPower,
	  GetTextForRadians           = GetTextForRadians,
	  GetTextForSine              = GetTextForSine,
	  GetTextForHyperbolicSine    = GetTextForHyperbolicSine,
	  GetTextForSquareRoot        = GetTextForSquareRoot,
	  GetTextForTangent           = GetTextForTangent,
	  GetTextForHyperbolicTangent = GetTextForHyperbolicTangent,
	}

  -- -------------------------------------------------------------------------
  -- Method new_TModelCalculator
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
  function new_TModelCalculator()
	
    local objModelCalculator =
    {
      GetType                     = GetType,
	  Reset                       = Reset,
	  AppendDecimalPoint          = AppendDecimalPoint,
	  AppendDigit0                = AppendDigit0,
	  AppendDigit1                = AppendDigit1,
	  AppendDigit2                = AppendDigit2,
	  AppendDigit3                = AppendDigit3,
	  AppendDigit4                = AppendDigit4,
	  AppendDigit5                = AppendDigit5,
	  AppendDigit6                = AppendDigit6,
	  AppendDigit7                = AppendDigit7,
	  AppendDigit8                = AppendDigit8,
	  AppendDigit9                = AppendDigit9,
      ToggleSign                  = ToggleSign,	  
	  GetPlusSign                 = GetPlusSign,
	  GetMinusSign                = GetMinusSign,
	  GetMultiplySign             = GetMultiplySign,
	  GetDivideSign               = GetDivideSign,
	  GetPowerSign                = GetPowerSign,
	  GetPlusMinus                = GetPlusMinus,
	  tblGetTexts                 = objGetTexts,
	  SetValue                    = SetValue,
	  Add                         = Add,
	  Subtract                    = Subtract,
	  Multiply                    = Multiply,
	  Divide                      = Divide,
	  Power                       = Power,
	  AbsoluteValue               = AbsoluteValue,
	  ArcCosine                   = ArcCosine,
	  ArcSine                     = ArcSine,
	  ArcTangent                  = ArcTangent,
	  ArcTangent2                 = ArcTangent2,
	  Ceiling                     = Ceiling,
	  Cosine                      = Cosine,
	  HyperbolicCosine            = HyperbolicCosine,
	  Degrees                     = Degrees,
	  Exp                         = Exp,
	  Floor                       = Floor,
	  FMod                        = FMod,
	  NaturalLogarithm            = NaturalLogarithm,
	  Base10Logarithm             = Base10Logarithm,
	  Maximum                     = Maximum,
	  Minimum                     = Minimum,
	  Pi                          = Pi,
	  Radians                     = Radians,
	  Sine                        = Sine,
	  HyperbolicSine              = HyperbolicSine,
	  SquareRoot                  = SquareRoot,
	  Tangent                     = Tangent,
	  HyperbolicTangent           = HyperbolicTangent,
	  Evaluate                    = Evaluate,
	  EvaluateInfix               = EvaluateInfix,
	  tblStack                    = new_TStack(),
	  strValue                    = "",
    }

    return objModelCalculator
  end
end  

